clear all;
close all;

rng(1)
% 2-dim toy data generation
C=5;
samples_per_class=500;
sigma=1.0;
num_samples = C*samples_per_class;

% classes A-E with negative labels
noise_dim = 4;
feature_dim = 2;
dim = noise_dim+feature_dim;

cluster_centers = [2  -4; -3  3; 3   1; -5 -2; -4 -5];
colors_samples = {'og','sr','xb','+m','dy'};
colors_classifiers = {'g','r','b','m','y'};

data = zeros(dim+1, num_samples);
for j=1:C
    data(:, (j-1)*samples_per_class+1:j*samples_per_class) = ...
        [repmat(cluster_centers(j, :),samples_per_class,1) + randn(samples_per_class,2)*sigma ...
        randn(samples_per_class, noise_dim)*3 ...
        j*ones(samples_per_class, 1)]';
end

% Suffle data
shuffle = randperm(size(data, 2));
data = data(:, shuffle);

% Split data into training and test set
N = num_samples-0.1*num_samples;
train_data = data(:, 1:N);
test_data = data(:, N+1:end);

figure;
subplot(1,2,1);
hold on
for j=1:C
   scatter(train_data(1, train_data(end, :) == j),...
   train_data(2, train_data(end, :) == j),colors_samples{j});
end
axis equal;
title('Training Data');

subplot(1,2,2);
hold on
for j=1:C
   scatter(test_data(1, test_data(end, :) == j),...
   test_data(2, test_data(end, :) == j),colors_samples{j});
end
axis equal;
title('Test Data');


X = train_data(1:dim, :)';
%Normalize data
X = X/max(X(:));
labels = train_data(end, :)';

%%
lambda = 15;
% initialize variables
W = zeros(dim, C);
B = zeros(1, C);

figure;

for c=1:C
    y = 2*(labels==c)-1;
    
    [w, b, obj_val, pd_gap] = consensus_pdhg(X, y, lambda, 800, 2e-3);
    W(:, c) = w;
    B(c) = b;
    
    subplot(C, 2, (c-1)*2+1);
    plot(obj_val); 
    title(sprintf('Objective value class %d', c));
    subplot(C, 2, c*2);
    plot(pd_gap);
    title(sprintf('Primal-Dual-Gap class %d', c));
end



%% Plot classifiers on test data
classifier = @(W, B, X) X*W + repmat(B, size(X, 1), 1);
s = 0.1;
[x1Grid, x2Grid] = meshgrid(-4:s:4,...
    -4:s:4);
xGrid = [x1Grid(:),x2Grid(:)];
scores_train = classifier(W(1:2,:), B, xGrid);
figure;
hold on
for j=1:C
    scores_test = classifier(W, B, test_data(1:dim,:)');
    [~, predicted_classes_test] = max(scores_test');
    
    scatter(test_data(1, predicted_classes_test==j),...
        test_data(2, predicted_classes_test==j),colors_samples{j});
    
    [~,h] = contour(x1Grid,x2Grid,reshape(scores_train(:, j),size(x1Grid)),[0 0],'k');
    h.LineColor = colors_classifiers{j};   
end
drawnow
title('Predicted classes for test data and classifiers')